ChatParser = class(Turbine.UI.Window);

function ChatParser:Constructor()
    Turbine.UI.Window.Constructor(self);

    self.rules = {};
    
    self.parse = function(sender, args)
        self:Parse(args.ChatType, args.Message);
    end

    AddCallback(Turbine.Chat, "Received", self.parse);
    
    self.player = Turbine.Gameplay.LocalPlayer.GetInstance();
    self.playerName = self.player:GetName();
    AddCallback(self.player, "TargetChanged", function()
        self.target = self.player:GetTarget();
        if (self.target) then
            self.targetName = self.target:GetName();
        else
            self.targetName = nil;
        end
    end);
    self.player:TargetChanged();
end

function ChatParser:AddFilter(tabCard, filterSpec)
    for c = 1, #filterSpec, 1 do
        local channel, filterFunc = unpack(filterSpec[c]);
        if (not self.rules[channel]) then
            self.rules[channel] = {};
        end
        table.insert(self.rules[channel], { filterFunc, tabCard });
    end
end

function ChatParser:Parse(channel, message)
    local rules = self.rules[channel];
    if (not rules) then
        return;
    end

    -- Prepare the message for parsing: remove EOL chars.
    message = string.gsub(message, "%c$", "");
--to do: remove colors

    for r = 1, #rules, 1 do
        local filterFunc, tabCard = unpack(rules[r]);
        local outMessage = filterFunc(channel, message, self.playerName, self.targetName);
        if (outMessage) then
            tabCard:AddLine(outMessage);
        end
    end
end
