ChatTabCard = class(Turbine.UI.Control);

function ChatTabCard:Constructor()
    Turbine.UI.Control.Constructor(self);
    self:SetMouseVisible(false);

    self.tabHeight = 26;
    self.scrollBarSize = 10;
    self.indent = 0;
    self.maxLines = 200;
    self.lineSize = {};
   
    self.tab = ChatTab();
    self.tab:SetParent(self);

    -- Highlight the tab if card is not already in front.
    self.tab.MouseEnter = function(ctl)
        ctl:SetHighlight(true);
    end
    self.tab.MouseLeave = function(ctl)
        ctl:SetHighlight(self:IsInFront() and self.stack:IsInFront());
    end
    
    -- Enable the window to be moved by dragging the tab.
    self.tab.MouseDown = function(ctl, args)
        if (args.Button == Turbine.UI.MouseButton.Left) then
            ctl.mouseDown = true;
            ctl.originalMouseX, ctl.originalMouseY = Turbine.UI.Display.GetMouseX(), Turbine.UI.Display.GetMouseY();
        end
        self.stack:Activate();
    end
    self.tab.MouseUp = function(ctl, args)
        if (args.Button == Turbine.UI.MouseButton.Left) then
            ctl.mouseDown = false;
        end
    end
    self.tab.MouseMove = function(ctl)
        if (ctl.mouseDown) then
            local newMouseX, newMouseY = Turbine.UI.Display.GetMouseX(), Turbine.UI.Display.GetMouseY();
            local deltaX, deltaY = newMouseX - ctl.originalMouseX, newMouseY - ctl.originalMouseY;
            ctl.originalMouseX, ctl.originalMouseY = newMouseX, newMouseY;
            local left, top = ctl:GetPosition();
            ctl:SetPosition(left + deltaX, top + deltaY);
            left, top = self.stack:GetPosition();
            self.stack:SetPosition(left + deltaX, top + deltaY);
        end
    end

    self.border = Turbine.UI.Control();
    self.border:SetParent(self);
    self.padding = 1;

    self.text = Turbine.UI.Label();
    self.text:SetParent(self);
    self.text:SetPosition(self.padding, self.padding);
    self.text:SetMarkupEnabled(true);
    self.text:SetFont(Turbine.UI.Lotro.Font.Verdana16);
    self.text:SetForeColor(Turbine.UI.Color.PaleGoldenrod);
    self.text:SetSelectable(true);

    self.vScrollBar = Turbine.UI.Lotro.ScrollBar();
    self.vScrollBar:SetOrientation(Turbine.UI.Orientation.Vertical);
    self.vScrollBar:SetParent(self);
    self.vScrollBar:SetWidth(self.scrollBarSize);
    self.text:SetVerticalScrollBar(self.vScrollBar);
    self:BringToFront();
    self:DoLayout();
end

function ChatTabCard:SetStack(stack)
    self.stack = stack;
end

function ChatTabCard:GetTabControl()
    return self.tab;
end

function ChatTabCard:BringToFront()
    self.tab:SetHighlight(true);
    self.border:SetVisible(true);
    self.text:SetVisible(true);
    self.vScrollBar:SetParent(self);
    self:DoLayout();
end    

function ChatTabCard:SendToBack()
    self.tab:SetHighlight(false);
    self.text:SetVerticalScrollBar(nil);
    self.vScrollBar:SetParent(nil);
    self.border:SetVisible(false);
    self.text:SetVisible(false);
end

function ChatTabCard:IsInFront()
    return (self.text:IsVisible());
end

function ChatTabCard:SetTabText(text)
    self.tabText:SetText(text);
end

function ChatTabCard:GetTabText()
    return self.tabText:GetText();
end

function ChatTabCard:SetBackColor(color)
    self.border:SetBackColor(color);
    self.text:SetBackColor(color);
end

function ChatTabCard:DoLayout()
    self.tab:SetIndent();
    local parent = self:GetParent();
    if (parent) then
        local left, top = parent:PointToScreen(self:GetPosition());
        self.tab:SetLeft(left - self.tabHeight);
    end

    local width, height = self:GetSize();
    self.border:SetSize(width, height);
    self.vScrollBar:SetLeft(width - self.scrollBarSize);
    self.vScrollBar:SetHeight(height);
    self.text:SetSize(width - (self.padding + self.scrollBarSize), height - self.padding);
end

function ChatTabCard:SetSize(width, height)
    Turbine.UI.Control.SetSize(self, width, height);
    self:DoLayout();
end

function ChatTabCard:SetWidth(width)
    Turbine.UI.Control.SetWidth(self, width);
    self:DoLayout();
end

function ChatTabCard:SetHeight(height)
    Turbine.UI.Control.SetHeight(self, height);
    self:DoLayout();
end

function ChatTabCard:AddLine(text)
    if (#self.lineSize >= self.maxLines) then
        self.text:SetSelection(1, self.lineSize[1]);
        self.text:SetSelectedText(nil);
        table.remove(self.lineSize, 1);
    end
    local len = string.len(text);
    if (#self.lineSize > 0) then
        text = "\n" .. text;
    end
    table.insert(self.lineSize, len + 1);
    self.text:AppendText(text);
end
