ChatTabCardStack = class(Turbine.UI.Lotro.Window);

function ChatTabCardStack:Constructor()
    Turbine.UI.Lotro.Window.Constructor(self);
    self:SetVisible(true);
    self:SetResizable(true);
    
    self.grip = Turbine.UI.Control();
    self.grip:SetBackColor(Turbine.UI.Color(0, 0, 0 ,0));
--self.grip:SetBackColor(Turbine.UI.Color(1, 1, 1 ,0));
    self.grip:SetParent(self);
    self.grip:SetMouseVisible(false);
    
    self.interior = Turbine.UI.Control();
    self.interior:SetParent(self.grip);
    self.interior:SetPosition(2, 2);

    self.line = Turbine.UI.Control();
    self.line:SetParent(self.grip);
    self.line:SetBackColor(Turbine.UI.Color(1, 43/255, 42/255, 42/255));
    
    self.commands = Turbine.UI.Control();
    self.commands:SetParent(self.grip);

    self.resizer = Turbine.UI.Control();
    self.resizer:SetParent(self.grip);
    self.resizer:SetMouseVisible(false);
    self.resizer:SetSize(16, 20);
    self.resizer.icon = Turbine.UI.Control();
    self.resizer.icon:SetParent(self.resizer);
    self.resizer.icon:SetPosition(0, 3);
    self.resizer.icon:SetMouseVisible(false);
    self.resizer.icon:SetBackground(0x4100013D);
    self.resizer.icon:SetBlendMode(Turbine.UI.BlendMode.Overlay);

    self.tabCards = {};
    self:SetBackColor(Turbine.UI.Color(0, 0, 0, 0));
    
    -- Default settings:
    self.settings = {};
    self.settings.position = {100, 100};
    self.settings.size = {400, 200};
    self.settings.language = Turbine.Language.Russian;
    self.settings.pluginVersion = tonumber(Plugins.ChatFilters:GetVersion());

    self.noSave = 3;
    self:ApplySettings();
    
    self:LoadSettings();
    self:Activate();
    self:SetWantsUpdates(true);
end

function ChatTabCardStack:LoadSettings()
    Turbine.PluginData.Load(Turbine.DataScope.Character, "RussianChatFilters", function(loadStr)
        if (loadStr) then
            -- Workaround for Turbine localization bug -- Thanks, Lynx3d!
            local settings = ImportTable(loadStr);
            if (not settings) then
                Turbine.Shell.WriteLine("Failed to parse RussianChatFilters.plugindata file!");
                return;
            end
        
            -- Previously-saved settings override the defaults
            DeepTableCopy(settings, self.settings);
            
            self:ApplySettings();
        end
    end);
end

function ChatTabCardStack:ApplySettings()
    self.noSave = self.noSave + 1;
    L:SetLanguage(self.settings.language);
    
    local left, top = unpack(self.settings.position);
    self:SetPosition(left, top);
    
    local width, height = unpack(self.settings.size);
    self:SetSize(width, height);
    
    self.selectedCommand = self.settings.selectedCommand;
    if (self.commandTable) then
        self:SetCommands(self.commandTable);
    end
end

function ChatTabCardStack:SaveSettings()
    if (self.noSave > 0) then
        self.noSave = self.noSave - 1;
        return;
    end
    
    -- Workaround for Turbine localization bug -- Thanks, Lynx3d!
    local saveData = ExportTable(self.settings);
    Turbine.PluginData.Save(Turbine.DataScope.Character, "RussianChatFilters", saveData, function()
--Puts("Save complete.");
    end);
end

function ChatTabCardStack:AddTabCard(tabCard)
    table.insert(self.tabCards, tabCard);
    tabCard:SetParent(self.interior);
    tabCard:SetStack(self);
    self:BringToFront(tabCard);
    tabCard:SetBackColor(self:GetBackColor());

    -- Default behavior when the tab is clicked is to bring the card to the front.
    tabCard.Click = function(sender, args)
        if (args.Button == Turbine.UI.MouseButton.Left) then
            self:BringToFront(sender);
        end
    end
end

function ChatTabCardStack:BringToFront(tabCard)
    for t = 1, #self.tabCards do
        local otherTabCard = self.tabCards[t];
        if (otherTabCard == tabCard) then
            tabCard:BringToFront();
            tabCard:SetSize(self.interior:GetSize());
            self.frontTabCard = tabCard;
        else
            otherTabCard:SendToBack();
        end
    end
end

function ChatTabCardStack:SetBackColor(color)
    for t = 1, #self.tabCards do
        self.tabCards[t]:SetBackColor(color);
    end
    self.commands:SetBackColor(color);
    self.resizer:SetBackColor(color);
end

function ChatTabCardStack:GetBackColor()
    return self.commands:GetBackColor();
end

function ChatTabCardStack:SetWidth(width)
    self:SetSize(width, self:GetHeight());
end

function ChatTabCardStack:SetHeight(height)
    self:SetSize(self:GetWidth(), height);
end

function ChatTabCardStack:PositionChanged()
    local left, top = self:GetPosition();
    self.settings.position = {left, top};
    self:SaveSettings();
end

function ChatTabCardStack:SizeChanged()
    local width, height = self:GetSize();
    self.settings.size = {width, height};
    self:SaveSettings();
    
    self.grip:SetSize(width, height);
    self.interior:SetSize(width - 4, height - 25);
    self.line:SetSize(width - 4, 1);
    self.line:SetPosition(2, height - 23);
    self.commands:SetSize(width - 20, 20);
    self.commands:SetPosition(2, height - 22);
    if (self.cmdDropDown) then
        local buttonWidth = self.cmdSendButton:GetWidth();
        local cmdDropDownWidth = math.floor((width - buttonWidth - 17) / 2);
        self.cmdDropDown:SetWidth(cmdDropDownWidth);
        if (cmdDropDownWidth < 360) then
            self.cmdDropDown:SetExpandedWidth(360);
        end
        self.cmdSendButton:SetLeft(width - buttonWidth - 24);
        self.hider:SetLeft(width - buttonWidth - 28);
        self.cmdQuickSlot:SetLeft(width - buttonWidth - 28);
        self.cmdTextBox:SetLeft(4 + 17 + cmdDropDownWidth);
        self.cmdTextBox:SetWidth(width - 17 - cmdDropDownWidth - buttonWidth - 30);
        
    end
    self.resizer:SetPosition(width - 18, height - 22);
    if (self.frontTabCard) then
        self.frontTabCard:SetSize(self.interior:GetSize());
    end
end

function ChatTabCardStack:Activated()
    self:SetZOrder(1);
    self:SetOpacity(1);
end

-- Temporary solution.  Will make a "chat tab overlay sensor" in next version.
function ChatTabCardStack:Update()
    self:Deactivated();
end

function ChatTabCardStack:Deactivated()
    -- Assuming we're superimposed on the chat panel, we only want to go to the back if the
    -- user clicked on the tabs of the standard chat panel.
    local mouseX, mouseY = Turbine.UI.Display.GetMousePosition();
    local left, top = self:GetPosition();
    if ((mouseX > left) and (mouseX < left + self:GetWidth()) and (mouseY < top) and (mouseY > top - 20)) then
        -- User clicked on the tabs of the standard chat panel.
        self:SetZOrder(-1);
        self:SetOpacity(0);
    end
end

function ChatTabCardStack:IsInFront()
    return (self:GetZOrder() >= 1);
end

function ChatTabCardStack:SetZOrder(zOrder)
    Turbine.UI.Lotro.Window.SetZOrder(self, zOrder);
    for t = 1, #self.tabCards do
        local tab = self.tabCards[t]:GetTabControl();
        tab:SetZOrder(zOrder);
        tab:MouseLeave(); -- update highlight
    end
    if (self.frontTabCard) then
        self.frontTabCard:GetTabControl():SetZOrder(zOrder);
    end
end

function ChatTabCardStack:ReverseCommandDisplay()
    self.settings.commandDisplayReversed = not self.settings.commandDisplayReversed;
    self:SetCommands(self.commandTable);
    self:SaveSettings();
end

function ChatTabCardStack:GetSelectedCommand()
    local str = self.cmdDropDown:GetText();
    if (self.settings.commandDisplayReversed) then
        self.selectedCommand = string.match(str, "/(.*):");
    else
        self.selectedCommand = string.match(str, "\%(/(.*)%)");
    end
    return self.selectedCommand;
end    

function ChatTabCardStack:SetCommands(commandTable)
    self.commandTable = commandTable;
    local commandList, selectedStr = {};
    for name in keys(commandTable) do
        local command, str = commandTable[name];
        if (self.settings.commandDisplayReversed) then
            str = "/" .. command .. ": " .. name;
        else
            str = name .. " (/" .. command .. ")";
        end
        table.insert(commandList, str);
        if (command == self.selectedCommand) then
            selectedStr = str;
        end
    end
    table.sort(commandList);
    if (not selectedStr) then
        selectedStr = commandList[1];
    end

    if (not self.reversalButton) then
        self.reversalButton = Turbine.UI.Control();
        self.reversalButton:SetParent(self.commands);
        self.reversalButton:SetBackground("Thurallor/ChatFilters/Images/reversal_normal.tga");
        self.reversalButton:SetSize(19, 19);
        self.reversalButton:SetPosition(1, 0);
        self.reversalButton.MouseEnter = function(ctl)
            ctl:SetBackground("Thurallor/ChatFilters/Images/reversal_highlight.tga");
        end
        self.reversalButton.MouseLeave = function(ctl)
            ctl:SetBackground("Thurallor/ChatFilters/Images/reversal_normal.tga");
        end
        self.reversalButton.MouseClick = function()
            self:ReverseCommandDisplay();
        end
    
        self.cmdTextBox = Turbine.UI.TextBox();
        self.cmdTextBox:SetParent(self.commands);
        self.cmdTextBox:SetHeight(18);
        self.cmdTextBox:SetTop(4);
        self.cmdTextBox:SetMultiline(false);
        self.cmdTextBox:SetFont(Turbine.UI.Lotro.Font.Verdana12);
        self.cmdTextBox:SetSelectable(true);
        self.cmdTextBox.TextChanged = function(sender)
            self.cmdQuickSlot:SetShortcut(Turbine.UI.Lotro.Shortcut(Turbine.UI.Lotro.ShortcutType.Alias, sender:GetText()));
        end;
        
        self.cmdQuickSlot = Turbine.UI.Lotro.Quickslot();
        self.cmdQuickSlot:SetParent(self.commands);
        self.cmdQuickSlot:SetWidth(20);
        self.cmdQuickSlot:SetTop(-9);
        self.cmdQuickSlot.MouseEnter = function()
            self.cmdSendButton:SetBackground(0x4113F1BE);
        end
        self.cmdQuickSlot.MouseLeave = function()
            self.cmdSendButton:SetBackground(0x4113F1BC);
        end
        
        self.hider = Turbine.UI.Control();
        self.hider:SetParent(self.commands);
        self.hider:SetSize(20, 35);
        self.hider:SetTop(-9);
        self.hider:SetBackColor(self:GetBackColor());
        self.hider:SetMouseVisible(false);

        self.cmdSendButton = Turbine.UI.Label();
        self.cmdSendButton:SetParent(self.commands);
        self.cmdSendButton:SetBackground(0x4113F1BC);
        self.cmdSendButton:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend);
        self.cmdSendButton:SetZOrder(2);
        self.cmdSendButton:SetTop(2);
        self.cmdSendButton:SetSize(16, 16);
        self.cmdSendButton:SetFont(Turbine.UI.Lotro.Font.Verdana12);
        self.cmdSendButton:SetMouseVisible(false);
    end
    
    if (self.dropDown) then
        self.dropDown:SetParent(nil);
    end
    self.cmdDropDown = Thurallor.UI.DropDown(commandList, selectedStr);
    self.cmdDropDown:SetBorderColor(Turbine.UI.Color(1, 0.25, 0.25, 0.25));
    self.cmdDropDown:SetInteriorColor(self:GetBackColor());
    self.cmdDropDown:SetAlignment(Turbine.UI.ContentAlignment.MiddleLeft);
    self.cmdDropDown:SetFont(Turbine.UI.Lotro.Font.Verdana12);
    self.cmdDropDown:SetParent(self.commands);
    self.cmdDropDown:SetPosition(18, 0);
    self.cmdDropDown.ItemChanged = function()
        local command = self:GetSelectedCommand();
        self.cmdTextBox:SetText("/" .. command .. " ");
        self.cmdTextBox:Focus();
        self.cmdTextBox:TextChanged();
        self.settings.selectedCommand = command;
        self:SaveSettings();
    end
    self.cmdDropDown:ItemChanged({Text = self.cmdDropDown:GetText()});
    self:SizeChanged();
end
